function getRabbitMQBaseUrl() {
    rabbitmqIP=$(kubectl -n rabbitmq get svc rabbitmq -o jsonpath='{.spec.clusterIP}')
    rabbitmqPort=$(kubectl -n rabbitmq get svc rabbitmq -o jsonpath='{.spec.ports[?(@.name=="management")].port}')
    RABBITMQ_BASEURL="http://${rabbitmqIP}:${rabbitmqPort}/api"
}

function getRabbitMQCredentials() {
    rabbitmqUser=$(kubectl -n rabbitmq get secrets rabbitmq-default-user -o jsonpath='{.data.username}' | base64 -d)
    rabbitmqPassword=$(kubectl -n rabbitmq get secrets rabbitmq-default-user -o jsonpath='{.data.password}' | base64 -d)
    RABBITMQ_CREDENTIALS="${rabbitmqUser}:${rabbitmqPassword}"
}

function createRabbitMqQueueResource() {
    local vhostName exchangeName
    vhostName=$1
    exchangeName=$2
    # Create new vhost
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X PUT "${RABBITMQ_BASEURL}"/vhosts/${vhostName}
    # Create a new exchange
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X PUT "${RABBITMQ_BASEURL}"/exchanges/${vhostName}/${exchangeName}
}

function cleanRabbitMqQueueResource() {
    local vhostName exchangeName
    vhostName=$1
    exchangeName=$2
    # Delete exchange
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json}" -X DELETE "${RABBITMQ_BASEURL}"/exchanges/${vhostName}/${exchangeName}
    # Delete new vhost
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X DELETE "${RABBITMQ_BASEURL}"/vhosts/${vhostName}
}


function createQueusAndResource() {
    local vhostName exchangeName queueName
    vhostName=$1
    exchangeName=$2
    queueName=$3
    createRabbitMqQueueResource "${vhostName}" "${exchangeName}"
    # Create a new queue
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X PUT -d"{\"durable\":true, \"arguments\":{\"x-queue-type\": \"quorum\"}}" "${RABBITMQ_BASEURL}"/queues/${vhostName}/${queueName}
    # Create a new binding between exchange and a queue
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X POST -d'{"routing_key": "test_key"}' "${RABBITMQ_BASEURL}"/bindings/${vhostName}/e/${exchangeName}/q/${queueName}
}

function cleanQueusAndResource() {
    local vhostName exchangeName queueName
    vhostName=$1
    exchangeName=$2
    queueName=$3
    # Delete queue
    curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X DELETE "${RABBITMQ_BASEURL}"/queues/${vhostName}/${queueName}
    cleanRabbitMqQueueResource "${vhostName}" "${exchangeName}"
}

publishMessageUtil() {
    local vhostName queueName exchangeName message result fetchQueueName
    vhostName=$1
    queueName=$2
    exchangeName=$3
    message=$4

    PUBLISH_MESSAGE_STATUS="passed"
    result="false"
    # Check if queue is present or not
    fetchQueueName=$(curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X GET "${RABBITMQ_BASEURL}"/queues/${vhostName}/${queueName} | jq '. | .name' | tr -d '"')
    if [[ -n ${fetchQueueName} && ${fetchQueueName} == "${queueName}" ]]; then
        # Post a message to exchange
        curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X POST -d"{\"properties\":{\"delivery_mode\":2},\"routing_key\":\"test_key\",\"payload\":\"${message}\",\"payload_encoding\":\"string\"}" "${RABBITMQ_BASEURL}"/exchanges/${vhostName}/${exchangeName}/publish
        result="true"
    fi
    [[ "${result}" == "false" ]] && PUBLISH_MESSAGE_STATUS="failed"
}

consumeMessageUtil() {
    local vhostName queueName testMessage result fetchQueueName
    vhostName=$1
    queueName=$2
    testMessage=$3

    CONSUME_MESSAGE_STATUS="passed"
    result="false"
    # Check if queue is present or not
    fetchQueueName=$(curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X GET "${RABBITMQ_BASEURL}"/queues/${vhostName}/${queueName} | jq '. | .name' | tr -d '"')
    if [[ -n ${fetchQueueName} && ${fetchQueueName} == "${queueName}" ]]; then
        # Read message from the queue
        response=$(curl -u "${RABBITMQ_CREDENTIALS}" -H "content-type:application/json" -X POST -d"{\"count\":\"1\",\"ackmode\":\"ack_requeue_true\",\"encoding\": \"auto\" }" "${RABBITMQ_BASEURL}"/queues/${vhostName}/${queueName}/get)

        if [[ -n "${response}" && "${response}" != "[]" ]]; then
            message=$(echo $response | jq ".[] | .payload" | tr -d '"')
            if [[ "${message}" == "${testMessage}" ]]; then
                result="true"
            fi
        fi    
    fi
    
    [[ "${result}" == "false" ]] && CONSUME_MESSAGE_STATUS="failed"
}

# This method will create queue, publish test message, consume test message and delete resource
publishConsumedAndResetResource() {
    local vhostName exchangeName queueName testMessage
    vhostName=$1
    exchangeName=$2
    queueName=$3
    testMessage=$4

    PUBLISH_AND_CONSUME_MESSAGE_STATUS="passed"
    getRabbitMQBaseUrl
    getRabbitMQCredentials
    createQueusAndResource "${vhostName}" "${exchangeName}" "${queueName}"
    publishMessageUtil "${vhostName}" "${queueName}" "${exchangeName}" "${testMessage}_${validTestRailId}"

    if [[ "${PUBLISH_MESSAGE_STATUS}" == "passed" ]]; then
        consumeMessageUtil "${vhostName}" "${queueName}" "${testMessage}_${validTestRailId}"
    fi

    cleanQueusAndResource "${vhostName}" "${exchangeName}" "${queueName}"

    [[ "${PUBLISH_MESSAGE_STATUS}" == "failed" ||  "${CONSUME_MESSAGE_STATUS}" == "failed" ]] &&  PUBLISH_AND_CONSUME_MESSAGE_STATUS="failed"
}

# This method will create queue, publish test message and return
publishAndForgive() {
    local vhostName exchangeName queueName testMessage
    vhostName=$1
    exchangeName=$2
    queueName=$3
    testMessage=$4

    PUBLISH_AND_FORGIVE_MESSAGE_STATUS="passed"
    getRabbitMQBaseUrl
    getRabbitMQCredentials
    cleanQueusAndResource "${vhostName}" "${exchangeName}" "${queueName}"

    createQueusAndResource "${vhostName}" "${exchangeName}" "${queueName}"

    publishMessageUtil "${vhostName}" "${queueName}" "${exchangeName}" "${testMessage}"

    [[ "${PUBLISH_MESSAGE_STATUS}" == "failed" ]] && PUBLISH_AND_FORGIVE_MESSAGE_STATUS="failed"
}

consumeOldSavedMessage() {
    local vhostName queueName testMessage status
    vhostName=$1
    queueName=$2
    testMessage=$3
    CONSUME_OLD_MESSAGE_STATUS="passed"

    getRabbitMQBaseUrl
    getRabbitMQCredentials

    consumeMessageUtil "${vhostName}" "${queueName}" "${testMessage}"
    
    [[ "${CONSUME_MESSAGE_STATUS}" == "failed" ]] &&  CONSUME_OLD_MESSAGE_STATUS="failed"
}

function testPodCountAndClusterStatus() {
    local count replicaCount podCount validTestRailIds requestedTestRailIds pod_name statusMessage expectedStatus testMessage expectedMessagePublishStatus
    count=$1
    requestedTestRailIds=$2
    expectedStatus=$3
    expectedMessagePublishStatus=$4

    validTestRailIds=("1621297" "1621299" "1621300" "1621298")
    testMessage=test_message
    for validTestRailId in "${validTestRailIds[@]}"; do
        if [[ "${requestedTestRailIds}" =~ "${validTestRailId}" ]]; then
            result="false";
            tatusMessage=""
            
            replicaCount=$(fetchReplicaCount statefulset rabbitmq rabbitmq-server)
            if [[ -z $replicaCount && -z $count ]]; then
                result="true"
                appendOutput "${validTestRailId}" "${result}"
                return
            fi
            podCount=$(kubectl -n rabbitmq get pods --field-selector=status.phase=Running --output json | jq -j '.items | length')
            pod_name=$(kubectl -n rabbitmq get pods --field-selector=status.phase=Running -o jsonpath="{.items[].metadata.name}")

            if [[ ! -z ${pod_name} && "${pod_name}" != " " ]]; then
                statusMessage=$(kubectl -n rabbitmq exec -it ${pod_name} -- rabbitmq-diagnostics ping | grep -i "${expectedStatus}" )
            fi

            if [[ ${podCount} -ge ${count} && ${podCount} -le ${replicaCount}  && ! -z ${statusMessage} && "${statusMessage}" != " " ]]; then
                result="true"
            fi

            affinity="\"topologyKey\":\"kubernetes.io/hostname\""
            pdbMaxUnavailable=$(kubectl -n rabbitmq get pdb rabbitmq -o jsonpath='{.spec.maxUnavailable}')
            pdbMinAvailable=$(kubectl -n rabbitmq get pdb rabbitmq -o jsonpath='{.spec.minAvailable}')
            podAffinity=$(kubectl -n rabbitmq get statefulset rabbitmq-server -o jsonpath='{.spec.template.spec.affinity}' | grep $affinity)

            [[ result="true" && -z ${pdbMaxUnavailable} ]] && result="false"

            [[ result="true" && -n ${pdbMinAvailable} ]] && result="false"

            [[ result="true" && -z ${podAffinity} ]] && result="false"

            if [[ "${result}" == "true" ]]; then
                publishConsumedAndResetResource "testvhost" "textexchange" "testqueue" "${testMessage}_${validTestRailId}"
                [[ "${PUBLISH_AND_CONSUME_MESSAGE_STATUS}" == "passed" && "${expectedMessagePublishStatus}" == "false" ]] && result="false"
                [[ "${PUBLISH_AND_CONSUME_MESSAGE_STATUS}" == "failed" && "${expectedMessagePublishStatus}" == "true" ]] && result="false"
            fi
            appendOutput "${validTestRailId}" "${result}"
        fi
    done
}

function testConsumerStatus() {
    local validTestRailIds requestedTestRailIds expectedStatus testMessage
    requestedTestRailIds=$1
    expectedStatus=$2
    validTestRailIds=("1628695" "1628694")

    for validTestRailId in "${validTestRailIds[@]}"; do
        if [[ "${requestedTestRailIds}" =~ "${validTestRailId}" ]]; then
            result="false";

            consumeOldSavedMessage "persistenttestvhost" "persistenttestqueue" "persistent_testmessage"

            [[ "${CONSUME_MESSAGE_STATUS}" == "passed" && "${expectedStatus}" == "true" ]] && result="true"
            
            [[ "${CONSUME_MESSAGE_STATUS}" == "failed" && "${expectedStatus}" == "false" ]] && result="true"
            
            appendOutput "${validTestRailId}" "${result}"
        fi
    done
}

function setRabbitmqValidationParam() {
    local validationStage
    validationStage=$1
    publisherAndConsumerStatus="true"
    consumerStatus="true"
    clusterStatus="Ping succeeded"
    case "$validationStage" in
    3nodeup)
        minCount=3
        testRailIds="1621297"
        ;;
    2nodeup)
        minCount=2
        testRailIds="1621299,1628695"
        ;;
    1nodeup|0nodeup)
        minCount=""
        publisherAndConsumerStatus="false"
        consumerStatus="false"
        testRailIds="1621300"
        ;;
    rebootAllServers)
        minCount=3
        testRailIds="1621298,1628694"
        ;;
    *)
        echo "Unexpected option ${validationStage} passed to ${FUNCNAME[0]}"
        exit 1
        ;;
    esac    
}

function executeRabbitMQTest() {

    setRabbitmqValidationParam "${VALIDATE_STAGE}"

    testPodCountAndClusterStatus "${minCount}" "${testRailIds}" "${clusterStatus}" "${publisherAndConsumerStatus}"

    publishAndForgive "persistenttestvhost" "persistenttestexchange" "persistenttestqueue" "persistent_testmessage"

    testConsumerStatus "${testRailIds}" "${consumerStatus}"

}
